# THIS FILE IS GENERATED FROM ENSTALLER SETUP.PY
version = '4.8.13'
full_version = '4.8.13'
git_revision = '7ba086933768569058f8d800a8e487e7c341f48b'
is_released = True

if not is_released:
    version = full_version
